;;  PVoice.s written by Ed Mandy (emandy@atcorp.com)
;;     Much of this code has been taken from other sources
;;		  - www.bung.com.hk
;;		  - http://www.tripmode.com/gbdev/dl/source_code/asm/pocketvoice.txt

	.area	_CODE

   ;;
   ;; Play a sound clip from the PocketVoice
   ;;
	;; Entry conditions
	;;   DE = pointer to RAM buffer that contains sound to play
   ;;   BC = num of bytes to play
	;;
	;; Register used: AF, DE, BC
   ;;
   ;; Taken from (Bung's) on-line documentation
   ;; To initialize the sound chip:
   ;;   Write 0x80 to 0xA000 before you use the sound chip.
   ;;   The byte rate is about 3kB per second.
	;; To start playing sound:
	;;	  Write 0x81 to 0xA000 and then sound data has to be written to 0xA001.
	;;	To stop playing sound:
	;;	  Write 0x84 to 0xA000 and delay 10ms (about 1 frame period),
   ;;   then write 0x80 to 0xA000.
.playVoice::
      ; Now make sure BC != 0
      ld    a,b
      or    c
      jr z,.quitPlayVoice  ;bc == 0
      ;
.startPlay:
		push de
		push bc
         call WeirdRoutine
      pop bc
      pop de
		ld	a,#0x81
		ld	(0xa000),a
.playloop:
		ld	a,(0xa000)
		bit	3,a		; d3=1 for ready to write a new sound data
		jr	z,.playloop	; about 330us per byte
		ld	a,(de)		; de pointing to Ram buffer
		ld	(0xa001),a
      inc   DE
		;
      dec   BC
      ld    a,b
      or    c
      jr    nz,.playloop ;bc!=0
		;
      push de
      call StopSound
      pop  de
.quitPlayVoice:
      ret

   ;;
   ;; Record a sound clip to the PocketVoice
   ;;    Stop recording when you have recorded your max num of bytes
   ;;       or when the A button has been lifted
   ;;
	;; Entry conditions
	;;   DE = pointer to RAM buffer that contains sound to record
   ;;   BC = num bytes to record
	;;
	;; Register used: AF, DE
   ;;
   ;; Taken from (Bung's) on-line documentation
	;;	To start recording sound:
	;;		Write 0x82 to 0xA000 and sound data can be read from 0xA001.
	;;	To stop recording sound:
	;;		Write 0x80 to 0xA000.
.recVoice::
      ld    a,b
      or    c
      jr z,.quitRecVoice  ;bc == 0
      ;
      call .joypad  ;returns joypad bits in A
      and    #0x10   ;test for J_A
      jr    z,.quitRecVoice ; J_A is not down, don't record
      ;
.startRec:
      ld	a,#0x82
      ld	(0xa000),a
.recloop:
      ;
      ld	a,(0xa000)
      bit	3,a		; d3=1 for data ready to be read
      jr	z,.recloop	; about 330us per byte
      ld	a,(0xa001)
      ld	(de),a		; de pointing to Ram buffer
      inc   de
      ;
      dec   BC
      ld    a,b
      or    c
      jr    z,.stopRecVoice ;bc==0
      call .joypad  ;returns joypad bits in A
      and    #0x10   ;test for J_A
      jr    z,.stopRecVoice ;J_A is not still down
      ;
      jr    .recloop
      ;
.stopRecVoice:
      ld	a,#0x80
      ld	(0xa000),a
.quitRecVoice:
      ret


   ;; Init PVoice
_initPVoice::
	;; Init sound
	ld	a,#0x80
	ld	(0xa000),a
   RET

	;; Record/Play Voice
	;;
	;; Entry conditions
	;;   DE = pointer to RAM Buffer
   ;;   BC = max number of bytes to record/play
	;;
	;; Registers used: AF, DE

_recVoice::
	PUSH	BC
   LDA   HL,7(SP) ; Skip return address and registers
	LD	B,(HL)		; BC = max number of bytes to record
	DEC	HL
	LD	C,(HL)
	DEC	HL
	LD	D,(HL)		; DE = ROM Address to record to
	DEC	HL
	LD	E,(HL)
   PUSH DE
	CALL	.recVoice
   POP  BC
   ;
   LD A,E         ; do DE=newDE-oldDE (or DE-BC)
   SUB  C         ; E = A - C where A = E
   LD E,A
   LD A,D
   SBC  B         ; Sub and use the carry flag
   LD D,A         ; D = A - B where A = D
   ;
	POP	BC
	RET

   ;;Rec and Play don't return the num bytes correctly

_playVoice::
	PUSH	BC
	LDA	HL,7(SP)	; Skip return address and registers
	LD	B,(HL)		; BC = number of bytes to play
	DEC	HL
	LD	C,(HL)
	DEC	HL
	LD	D,(HL)		; DE = ROM Address to play from
	DEC	HL
	LD	E,(HL)
   PUSH	DE       ; push start address
	CALL	.playVoice
   POP   BC       ; pop start address (oldDE) to BC DE=End Addr
   ;
   LD A,E         ; do DE=newDE-oldDE (or DE-BC)
   SUB  C         ; E = A - C where A = E
   LD E,A
   LD A,D
   SBC  B         ; Sub and use the carry flag
   LD D,A         ; D = A - B where A = D
   ;
	POP	BC
	RET

; WeirdRoutine (aka .trigerplay from Bung) From Joshua Neal <tinara@goodnet.com>
;  See http://www.gbdev.org/news/dl/source_code/asm/pocketvoice.txt

.trigerplay:
WeirdRoutine::
        ld      de,#0x200
        ld      a,#0x82
.loop0:
        ld      (0xA000),a
        xor     #0x20
        ld      b,a
        push    de
        ld      de,#0x18
        call    Delay1
        pop     de
        dec     de
        ld      a,e
        or      d
        ld      a,b
        jr      nz,.loop0
        call    StopSound
        ld      de,#0x600
        call    Delay1
        ret

StopSound::
        ld      a,#0x84
        ld      (0xA000),a
        ld      de,#0x600
        call    Delay1
        ld      a, #0x80
        ld      (0xA000),a
        ret

Delay1::
.loop1:
		  dec     de
        ld      a,e
        cp      #0
        jr      nz,.loop1
        ld      a,d
        cp      #0
        jr      nz,.loop1
        ret

; end WeirdRoutine From Joshua Neal <tinara@goodnet.com>


;; Joypad routine from GBDK
	;; Get Keypad Button Status
	;; The following bits are set if pressed:
	;;   0x80 - Start   0x08 - Down
	;;   0x40 - Select  0x04 - Up
	;;   0x20 - B	    0x02 - Left
	;;   0x10 - A	    0x01 - Right
.joypad::
	PUSH	BC		; Save modified registers
	LD	A,#0x20
	LDH	(0),A		; Turn on P15

	LDH	A,(0)		; Delay
	LDH	A,(0)
	CPL
	AND	#0x0F
	SWAP	A
	LD	B,A
	LD	A,#0x10
	LDH	(0),A		; Turn on P14
	LDH	A,(0)		; Delay
	LDH	A,(0)
	LDH	A,(0)
	LDH	A,(0)
	LDH	A,(0)
	LDH	A,(0)
	CPL
	AND	#0x0F
	OR	B
	SWAP	A
	LD	B,A
	LD	A,#0x30
	LDH	(0),A		; Turn off P14 and P15 (reset joypad)
	LD	A,B
	POP	BC		; Restore registers
	RET
;; end Joypad routine from GBDK


;; Adapted from Bung's PVoice Info #2 by ERM

;; If you want to write data to Pocket Voice's Flash Rom, it needs to
;; copy some routines includes the writing Flash Rom main routine to
;; GB's work Ram 0xc000--0xdfff and jump to run it. It is because during
;; flashing, the whole Flash Rom can't be read as normal, some times it
;; just be read and checked in it's d7 or d6 only for waiting last page
;; written ready. So some routines need to be run in GB's work ram area.

old_PVROMcpy::
	PUSH	BC
	PUSH	DE

   LD BC, #0xD000 ;;where to copy it to
	CALL #copy_PVoiceROMcpy  ;copy the routine to RAM

   ; set up the registers
	LDA	HL,11(SP)	; Skip return address and registers
	LD	D,(HL)		; DE = number of bytes to copy
	DEC	HL
	LD	E,(HL)
	DEC	HL
   ;
	LD	B,(HL)		; BC = pointer to copy from
	DEC	HL
	LD	C,(HL)
	DEC	HL
   ;
	LD	A,(HL-)		; HL = pointer to copy to
	LD	L,(HL)
	LD	H,A

  	CALL 0xD000  ;;this better be the same as above

   POP DE
   POP BC
   RET

;;
;; *** NOTE:  This doesn't seem to work!!!!!!!!
;;    Before calling, please set these up
;; DE = number of bytes to copy
;; BC = pointer to copy from
;; HL = pointer to copy to
;;    and make sure that PVoiceROMcpy is in RAM
;;
PVoiceROMcpy::
   ;; Do the flash command
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
   ;
	ld	a,#2
	ld	(0x6000),a		; set winbond flash rom bank (to 2???)
	xor	a
	ld	(0x7000),a
   ;
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable
	ld	a,#0xaa
	ld	(0x7555),a		; write 0xaa to 0x1555
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
   ;
	ld	a,#1
	ld	(0x6000),a		; set winbond flash rom bank (to 1???)
   ;
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable
	ld	a,#0x55
	ld	(0x6aaa),a		; write 0x55 to 0x2aaa
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
   ;
	ld	a,#7
	ld	(0x6000),a		; set winbond flash rom bank (to 7???)
   ;
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable
   ;
	ld	a,#0xa0		; write flash command
	ld	(0x7555),a		; write command to winbond flash rom
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
   ;
	ld	a,#4			; ROM bank #4 [NOTE this should be variable]
	ld	(0x6000),a	; set current rom bank [NOTE this should be variable]
   ;
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable

   ;;do the copy
.copyLoop:  		;  I think that copyLoop is WRONG!!!
	LD	A,(BC)		; point to Gameboy work Ram buffer (one page 256 bytes)
	ld	(HL),a		; point to Flash Rom address which page you want to write
   INC   BC
	inc	HL
	;
   dec   DE
   ld    A,D
   or    E
   jr    nz,.copyLoop ;DE!=0
   ;
        ld      a,#0xC4
        ld      (0xA000),a
        ; delay
        ld      de,#0x0A20
.wrLoop1:
		  dec     de
        ld      a,e
        cp      #0
        jr      nz,.wrLoop1
        ld      a,d
        cp      #0
        jr      nz,.wrLoop1
        ; end delay
        ld	    a,#0xC0		; set gameboy rom bank I/O write mode
        ld      (0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
        ld      a, #0x80
        ld      (0xA000),a

   ;
	RET
end_PVoiceROMcpy::

;;
;; copy_PVoiceROMcpy
;; set BC to the RAM location that you want to copy PVoiceROMcpy to before
;; calling this
;;
copy_PVoiceROMcpy::
;copy PVoiceROMcpy into RAM (D000)
	LD HL, #PVoiceROMcpy
   LD DE, #end_PVoiceROMcpy
   ;
   LD A,E         ; do DE=end_of_set_PV_offset-set_PV_offset (or DE-HL)
   SUB  L         ; E = A - L where A = E
   LD E,A
   LD A,D
   SBC  H         ; Sub and use the carry flag
   LD D,A         ; D = A - H where A = D
   ;					; Now, DE = the number of bytes to copy to RAM(0xD000)
   ;LD BC, #0xD000

.copy_PVoiceROMcpyLoop1$:
	LD A,(HL+)
   LD (BC),A
   INC BC
   DEC DE
   LD A, E
   OR D
   JP NZ,.copy_PVoiceROMcpyLoop1$
	RET

;;
;;  *****  Begin PV offset setting code
;;
;;  Call setPVoffset(char offset) to set the offset and restart at that offset
;;
_ram_set_PV_offset::  ;old name
_setPVoffset::        ;new name
;01264 CDAF12   CALL 12AF	;copy it
	CALL #copy_set_PV_offset
;disregard this junk
;01267 CD8015   CALL 1580
;0126A 21E02F   LD   HL,2FE0
;0126D FAC3C7   LD   A,(C7C3)	;seems to be 0 most of the time
;01270 85       ADD  A,L
;01271 6F       LD   L,A
;01272 2A       HL++ A,(HL)
;01273 C300D0   JP   D000	;run the code at 0xD000
	LDA	HL,2(SP)	; Skip return address and registers
	LD		A,(HL)	; make sure A holds the offset to switch to
	JP 0xD000


set_PV_offset::
;;From pv2_0.gb
;015BC EA03A0   LD   (A003),A
	LD (#0xA003),A					 	;load the offset into (A003)
;015BF 3E03     LD   A,03
;015C1 EA0060   LD   (6000),A
	LD A,#3
	LD (#0x6000),A						;set the ROM bank (low byte) to 3
;015C4 AF       XOR  A
;015C5 EA0070   LD   (7000),A
	XOR A
   LD (#0x7000),A						;set the ROM bank (high byte) to 0

;015C8 F3       DI
	DI
;015C9 AF       XOR  A
;015CA E0FF     OUT  (FF),A
	XOR A
   LDH (#0xFF),A						;.IE		= 0xFF	; Interrupt enable
;015CC 3E0A     LD   A,0A
;015CE EA0000   LD   (0000),A
	LD A,#0x0A
   LD (#0x0000),A						;?? (SRAM Control)
;015D1 3E00     LD   A,00
;015D3 E040     OUT  (40),A
;015D5 3E01     LD   A,01
;015D7 E04F     OUT  (4F),A
	LD A, #0x00
   LDH (#0x40), A						;.LCDC		= 0x40	; LCD control
	LD A, #0x01
   LDH (#0x4F), A                ;.VBK		= 0x4F	; VRAM bank
;015D9 210080   LD   HL,8000
;015DC 110020   LD   DE,2000
	LD HL,#0x8000
   LD DE,#0x2000
;015DF F041     IN   A,(41)
;015E1 E602     AND  02
;015E3 20FA     JR   NZ,15DF
.15DF$:
	LDH A, (#0x41)			;	.STAT		= 0x41	; LCD status
   AND #0x02
   JR NZ, .15DF$
;015E5 3E00     LD   A,00
;015E7 22       HL++ (HL),A
;015E8 1B       DEC  DE
;015E9 7A       LD   A,D
;015EA B3       OR   E
;015EB 20F2     JR   NZ,15DF
	LD A, #0x00
   LD	(HL+),A
   DEC DE
   LD A, D
   OR E
   JR NZ,.15DF$
;015ED E04F     OUT  (4F),A
;015EF 210080   LD   HL,8000
;015F2 110020   LD   DE,2000
	LDH (0x4F),A				;.VBK		= 0x4F	; VRAM bank
   LD HL,#0x8000
   LD DE,#0x2000
;015F5 F041     IN   A,(41)
;015F7 E602     AND  02
;015F9 20FA     JR   NZ,15F5
.15F5$:
	LDH A,(0x41)             ;.STAT		= 0x41	; LCD status
   AND #0x02
   JR NZ, .15F5$
;015FB 3E00     LD   A,00
;015FD 22       HL++ (HL),A
;015FE 1B       DEC  DE
;015FF 7A       LD   A,D
;01600 B3       OR   E
;01601 20F2     JR   NZ,15F5
	LD A, #0x00
   LD (HL+),A
   DEC DE
   LD A, D
   OR E
   JR NZ, .15F5$
;01603 1800     JR   1605
	JR .1605$						;why?
;01605 3E91     LD   A,91
;01607 E040     OUT  (40),A
.1605$:
	LD A, #0x91
   LDH (0x40), A				;.LCDC		= 0x40	; LCD control
;01609 F041     IN   A,(41)
;0160B E601     AND  01
;0160D 28FA     JR   Z,1609
.1609$:
	LDH A, (0x41)				;.STAT		= 0x41	; LCD status
   AND #0x01
   JR Z, .1609$
;0160F F041     IN   A,(41)
;01611 E601     AND  01
;01613 20FA     JR   NZ,160F
.160F$:
	LDH A, (0x41)				;.STAT		= 0x41	; LCD status
   AND #0x01
   JR NZ, .160F$
;01615 31FEFF   LD   SP,FFFE
	LD SP, #0xFFFE				  ;clear the stack
;01618 FAFFC0   LD   A,(C0FF);usually 0
	LD A, #0x00					  ;seems to be useless
;0161B C30001   JP   0100
	JP 0x0100
end_of_set_PV_offset:

copy_set_PV_offset::
;copy set_PV_offset into RAM (D000)
;012AF 21BC15   LD   HL,15BC
;012B2 116200   LD   DE,0062
;012B5 0100D0   LD   BC,D000
	LD HL, #set_PV_offset
   LD DE, #end_of_set_PV_offset
   ;
   LD A,E         ; do DE=end_of_set_PV_offset-set_PV_offset (or DE-HL)
   SUB  L         ; E = A - L where A = E
   LD E,A
   LD A,D
   SBC  H         ; Sub and use the carry flag
   LD D,A         ; D = A - H where A = D
   ;					; Now, DE = the number of bytes to copy to RAM(0xD000)
   LD BC, #0xD000
;012B8 2A       HL++ A,(HL)
;012B9 02       LD   (BC),A
;012BA 03       INC  BC
;012BB 1B       DEC  DE
;012BC 7B       LD   A,E
;012BD B2       OR   D
;012BE C2B812   JP   NZ,12B8
.12B8$:
	LD A,(HL+)
   LD (BC),A
   INC BC
   DEC DE
   LD A, E
   OR D
   JP NZ,.12B8$
;012C1 C9       RET
	RET

;_set_PV_ROM_bank::
;	LDA	HL,2(SP)	; Skip return address and registers
;	LD		A,(HL)	; make sure A holds the bank to switch to
;	LD (#0x6000),A ;set the ROM bank (low byte)
;	XOR A
;   LD (#0x7000),A	;set the ROM bank (high byte) to 0
;	ret
;;
;;  *****  End PV offset setting code
;;





;;next try on ROM copy
;;much taken from from PV2.0
;;
;; Bungs Words
;; If you want to write data to Pocket Voice's Flash Rom, it needs to
;; copy some routines includes the writing Flash Rom main routine to
;; GB's work Ram 0xc000--0xdfff and jump to run it. It is because during
;; flashing, the whole Flash Rom can't be read as normal, some times it
;; just be read and checked in it's d7 or d6 only for waiting last page
;; written ready. So some routines need to be run in GB's work ram area.
;;
;;
;;
_flashROM::
   PUSH	BC
   PUSH	DE

   LD BC, #0xD000 ;;where to copy it to
   CALL copy_flashcomstuff  ;copy the routine to RAM

   ; set up the registers
	LDA	HL,10(SP)	; Skip return address and registers
	;LD	B,(HL)		; B = number of bytes to copy
	;DEC	HL
	LD	C,(HL)		; C = bank
	DEC	HL
   ;
	LD	D,(HL)		; DE = pointer to copy from
	DEC	HL
	LD	E,(HL)
	DEC	HL
   ;
	LD	A,(HL-)		; HL = pointer to copy to
	LD	L,(HL)
	LD	H,A

   PUSH DE
   PUSH HL
   POP DE
   POP HL	;;HL = copy from    DE = copy to

   CALL 0xD000  ;;this better be the same as above

   POP DE
   POP BC
   RET

;;
;; copy_flashcomstuff
;; set BC to the RAM location that you want to copy PVoiceROMcpy to before
;; calling this
;;
copy_flashcomstuff::
;copy stuff for flashcom into RAM (BC)
   LD HL, #.writepage
   LD DE, #.end_flashcom
   ;
   LD A,E         ; do DE=end_of_set_PV_offset-set_PV_offset (or DE-HL)
   SUB  L         ; E = A - L where A = E
   LD E,A
   LD A,D
   SBC  H         ; Sub and use the carry flag
   LD D,A         ; D = A - H where A = D
   ;		  ; Now, DE = the number of bytes to copy to RAM(0xD000)

.copy_flashcomstuff1$:
   LD A,(HL+)
   LD (BC),A
   INC BC
   DEC DE
   LD A, E
   OR D
   JR NZ, .copy_flashcomstuff1$
   RET


;;
;;  ; write 0x100 bytes from (inc hl) to (inc de) flash rom
;;  notes
;;  E should = 0
;;  C should be the bank (to write to???)
;;
;;  offset of 9D00 for JP and CALL
;;
.writepage:
;0279D 3EA0     LD   A,A0
;0279F CDB4C5   CALL C5B4		;;call .flashcom
   LD A, #0xA0


   ;call .flashcom
;;  C needs to be the bank to copy to
.flashcom:
   PUSH AF
   LD   A, #0x82
   LD   (0xA000), A
   LD   A, #0x02
   LD   (0x6000), A
   XOR  A
   LD   (0x7000),A

   LD   A, #0xC2
   LD   (0xA000),A
   LD   A, #0xAA
   LD   (0x7555),A
   LD   A, #0x82
   LD   (0xA000),A

   LD   A, #0x01
   LD   (0x6000),A
   LD   A, #0xC2
   LD   (0xA000),A
   LD   A, #0x55
   LD   (0x6AAA),A
   LD   A, #0x82
   LD   (0xA000),A

   LD   A, #0x02
   LD   (0x6000),A
   LD   A, #0xC2
   LD   (0xA000),A

   POP  AF
   LD   (0x7555),A
   LD   A, #0x82
   LD   (0xA000),A
   LD A, C        ;used to be: LD   A,(C7DC)    ; ld a, (.bankno)
   LD   (0x6000),A
   LD   A, #0xC2
   LD   (0xA000),A
;;end .flashcom

;027A2 2A       HL++ A,(HL)
;027A3 12       LD   (DE),A
;027A4 13       INC  DE
;027A5 7B       LD   A,E
;027A6 E603     AND  03
;027A8 C2A2C4   JP   NZ,C4A2  ;;jp	nz,1$
   LD A, #0x01
   LD B, A	; B used to be passed in

   LD A, #0x00
   LD C, A      ;only do it BC times (100 seems like the limit)
.27A2$:
   LD A, (HL+)
   LD (DE), A
   INC DE
   DEC BC
   ;LD A, E
   ;AND #0x03
   ;JR NZ, .27A2$
;;
;;  this is for writing from ROM or record from PVoice
;;
;027AB FAC1C7   LD   A,(C7C1) ;;A=.selectmode     0=record 1=writeFromROM
;027AE FE00     CP   00
;027B0 C2B6C4   JP   NZ,C4B6	;;jp	nz,2$
;027B3 CDBFC4   CALL C4BF     ;;call	.C4BF
   ;LD A, #1 ;;A=.selectmode     0=record 1=writeFromROM
   ;CP #0
   ;JR NZ, .27B6$
   ;NOP ;used to be: CALL .C4BF$
.27B6$:

;027B6 7B       LD   A,E
;027B7 FE00     CP   00       ;;if E!=0, then jp 1$ (go from [D][E] to [D+1][E=0])
;027B9 C2A2C4   JP   NZ,C4A2	;;jp nz,1$
;027BC C37BC5   JP   C57B		;;jp	.waitwinbondok
   ;LD A, E
   ;CP #0
   ;JR NZ, .27A2$
   ;
   ;-erm
   LD A, B
   OR C
   JR NZ, .27A2$
   ;

   ;JR .waitwinbondok  ;;will do the RET

.waitwinbondok:

   XOR  A
   ;
   ;;;;ld (.wrtimeoutcnt),a ; set time out counter =0000 (must be completed within 10ms)
   LD B, A	; used to be: LD   (C7D8),A 
   ;
   LD   A, #0xFF  ;used to be 0x02
   LD C, A	; used to be: LD   (C7D9),A ; ld (.wrtimeoutcnt+1),a ; about 17ms

   LD   HL, #0x7FFF
.waitwinbondok1:
   LD   A,(HL)
   XOR  (HL)
   ;AND  #0x40   ;I dunno why, but it works a lot better without this
   JR   Z, #.setCF

   ;LD   A,(C7C1)  ;ld a,(.selectmode); 0=record sound from mic, !=0 write from ROM
   ;CP   00
   ;JR   NZ, .notRead
   ;CALL .read1voicedata
;.notRead:
   LD A, B	; used to be: LD   A,(C7D8)
   SUB  #0x01
   LD B, A	; used to be: LD   (C7D8),A
   LD A, C	; used to be: LD   A,(C7D9)
   SBC  A, #0x00
   LD C, A	; used to be: LD   (C7D9),A
   JR   NC, .waitwinbondok1


.clrCF:
   SCF  
   CCF			; return with 'c'=0 for failure  
   RET 
 
.setCF:
   SCF  		; return with 'c'=1 for completed
   RET 

.end_flashcom:



;027BF FA00A0   LD   A,(A000)
;027C2 CB5F     BIT  3,A
;027C4 C2C9C4   JP   NZ,C4C9  ;;jp .C4C9 ????????????????????
;027C7 37       SCF
;027C8 C9       RET
;
;.read1voicedata:
;   LD A, (0xA000)
;   BIT 3, A
;   JR NZ, .27C9$
;   SCF
;   RET
;
;027C9 FA01A0   LD   A,(A001)
;027CC 02       LD   (BC),A
;027CD 03       INC  BC
;027CE CD13C5   CALL C513
;027D1 37       SCF
;027D2 3F       CCF
;027D3 C9       RET
;
;.27C9$:
;   LD A, (0xA001)
;   LD (BC),A
;   INC BC
;   CALL .2813$
;   SCF
;   CCF
;   RET
