;;  PVoice.s written by Ed Mandy (emandy@atcorp.com)
;;     Much of this code has been taken from other sources
;;		  - www.bung.com.hk
;;		  - http://www.tripmode.com/gbdev/dl/source_code/asm/pocketvoice.txt

	.area	_CODE

   ;;
   ;; Play a sound clip from the PocketVoice
   ;;
	;; Entry conditions
	;;   DE = pointer to RAM buffer that contains sound to play
   ;;   BC = num of bytes to play
	;;
	;; Register used: AF, DE, BC
   ;;
   ;; Taken from (Bung's) on-line documentation
   ;; To initialize the sound chip:
   ;;   Write 0x80 to 0xA000 before you use the sound chip.
   ;;   The byte rate is about 3kB per second.
	;; To start playing sound:
	;;	  Write 0x81 to 0xA000 and then sound data has to be written to 0xA001.
	;;	To stop playing sound:
	;;	  Write 0x84 to 0xA000 and delay 10ms (about 1 frame period),
   ;;   then write 0x80 to 0xA000.
.playVoice::
      ; Now make sure BC != 0
      ld    a,b
      or    c
      jr z,.quitPlayVoice  ;bc == 0
      ;
.startPlay:
		push de
		push bc
         call WeirdRoutine
      pop bc
      pop de
		ld	a,#0x81
		ld	(0xa000),a
.playloop:
		ld	a,(0xa000)
		bit	3,a		; d3=1 for ready to write a new sound data
		jr	z,.playloop	; about 330us per byte
		ld	a,(de)		; de pointing to Ram buffer
		ld	(0xa001),a
      inc   DE
		;
      dec   BC
      ld    a,b
      or    c
      jr    nz,.playloop ;bc!=0
		;
      push de
      call StopSound
      pop  de
.quitPlayVoice:
      ret

   ;;
   ;; Record a sound clip to the PocketVoice
   ;;    Stop recording when you have recorded your max num of bytes
   ;;       or when the A button has been lifted
   ;;
	;; Entry conditions
	;;   DE = pointer to RAM buffer that contains sound to record
   ;;   BC = num bytes to record
	;;
	;; Register used: AF, DE
   ;;
   ;; Taken from (Bung's) on-line documentation
	;;	To start recording sound:
	;;		Write 0x82 to 0xA000 and sound data can be read from 0xA001.
	;;	To stop recording sound:
	;;		Write 0x80 to 0xA000.
.recVoice::
      ld    a,b
      or    c
      jr z,.quitRecVoice  ;bc == 0
      ;
      call .joypad  ;returns joypad bits in A
      and    #0x10   ;test for J_A
      jr    z,.quitRecVoice ; J_A is not down, don't record
      ;
.startRec:
      ld	a,#0x82
      ld	(0xa000),a
.recloop:
      ;
      ld	a,(0xa000)
      bit	3,a		; d3=1 for data ready to be read
      jr	z,.recloop	; about 330us per byte
      ld	a,(0xa001)
      ld	(de),a		; de pointing to Ram buffer
      inc   de
      ;
      dec   BC
      ld    a,b
      or    c
      jr    z,.stopRecVoice ;bc==0
      call .joypad  ;returns joypad bits in A
      and    #0x10   ;test for J_A
      jr    z,.stopRecVoice ;J_A is not still down
      ;
      jr    .recloop
      ;
.stopRecVoice:
      ld	a,#0x80
      ld	(0xa000),a
.quitRecVoice:
      ret


   ;; Init PVoice
_initPVoice::
	;; Init sound
	ld	a,#0x80
	ld	(0xa000),a
   RET

	;; Record/Play Voice
	;;
	;; Entry conditions
	;;   DE = pointer to RAM Buffer
   ;;   BC = max number of bytes to record/play
	;;
	;; Registers used: AF, DE

_recVoice::
	PUSH	BC
   LDA   HL,7(SP) ; Skip return address and registers
	LD	B,(HL)		; BC = max number of bytes to record
	DEC	HL
	LD	C,(HL)
	DEC	HL
	LD	D,(HL)		; DE = ROM Address to record to
	DEC	HL
	LD	E,(HL)
   PUSH DE
	CALL	.recVoice
   POP  BC
   ;
   LD A,E         ; do DE=newDE-oldDE (or DE-BC)
   SUB  C         ; E = A - C where A = E
   LD E,A
   LD A,D
   SBC  B         ; Sub and use the carry flag
   LD D,A         ; D = A - B where A = D
   ;
	POP	BC
	RET

   ;;Rec and Play don't return the num bytes correctly

_playVoice::
	PUSH	BC
	LDA	HL,7(SP)	; Skip return address and registers
	LD	B,(HL)		; BC = number of bytes to play
	DEC	HL
	LD	C,(HL)
	DEC	HL
	LD	D,(HL)		; DE = ROM Address to play from
	DEC	HL
	LD	E,(HL)
   PUSH	DE       ; push start address
	CALL	.playVoice
   POP   BC       ; pop start address (oldDE) to BC DE=End Addr
   ;
   LD A,E         ; do DE=newDE-oldDE (or DE-BC)
   SUB  C         ; E = A - C where A = E
   LD E,A
   LD A,D
   SBC  B         ; Sub and use the carry flag
   LD D,A         ; D = A - B where A = D
   ;
	POP	BC
	RET

; WeirdRoutine (aka .trigerplay from Bung) From Joshua Neal <tinara@goodnet.com>
;  See http://www.gbdev.org/news/dl/source_code/asm/pocketvoice.txt

.trigerplay:
WeirdRoutine::
        ld      de,#0x200
        ld      a,#0x82
.loop0:
        ld      (0xA000),a
        xor     #0x20
        ld      b,a
        push    de
        ld      de,#0x18
        call    Delay1
        pop     de
        dec     de
        ld      a,e
        or      d
        ld      a,b
        jr      nz,.loop0
        call    StopSound
        ld      de,#0x600
        call    Delay1
        ret

StopSound::
        ld      a,#0x84
        ld      (0xA000),a
        ld      de,#0x600
        call    Delay1
        ld      a, #0x80
        ld      (0xA000),a
        ret

Delay1::
.loop1:
		  dec     de
        ld      a,e
        cp      #0
        jr      nz,.loop1
        ld      a,d
        cp      #0
        jr      nz,.loop1
        ret

; end WeirdRoutine From Joshua Neal <tinara@goodnet.com>


;; Joypad routine from GBDK
	;; Get Keypad Button Status
	;; The following bits are set if pressed:
	;;   0x80 - Start   0x08 - Down
	;;   0x40 - Select  0x04 - Up
	;;   0x20 - B	    0x02 - Left
	;;   0x10 - A	    0x01 - Right
.joypad::
	PUSH	BC		; Save modified registers
	LD	A,#0x20
	LDH	(0),A		; Turn on P15

	LDH	A,(0)		; Delay
	LDH	A,(0)
	CPL
	AND	#0x0F
	SWAP	A
	LD	B,A
	LD	A,#0x10
	LDH	(0),A		; Turn on P14
	LDH	A,(0)		; Delay
	LDH	A,(0)
	LDH	A,(0)
	LDH	A,(0)
	LDH	A,(0)
	LDH	A,(0)
	CPL
	AND	#0x0F
	OR	B
	SWAP	A
	LD	B,A
	LD	A,#0x30
	LDH	(0),A		; Turn off P14 and P15 (reset joypad)
	LD	A,B
	POP	BC		; Restore registers
	RET
;; end Joypad routine from GBDK


;; Adapted from Bung's PVoice Info #2 by ERM

;; If you want to write data to Pocket Voice's Flash Rom, it needs to
;; copy some routines includes the writing Flash Rom main routine to
;; GB's work Ram 0xc000--0xdfff and jump to run it. It is because during
;; flashing, the whole Flash Rom can't be read as normal, some times it
;; just be read and checked in it's d7 or d6 only for waiting last page
;; written ready. So some routines need to be run in GB's work ram area.

;; *** NOTE:  This doesn't seem to work!!!!!!!!

_start_PVoiceROMcpy::
_PVoiceROMcpy::
	PUSH	BC
	LDA	HL,9(SP)	; Skip return address and registers
	LD	D,(HL)		; DE = number of bytes to copy
	DEC	HL
	LD	E,(HL)
	DEC	HL
	LD	B,(HL)		; BC = pointer to copy from
	DEC	HL
	LD	C,(HL)
	DEC	HL
	LD	A,(HL-)		; HL = pointer to copy to
	LD	L,(HL)
	LD	H,A

   push DE

   ;; Do the flash command
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
	ld	a,#2
	ld	(0x6000),a		; set winbond flash rom bank
	xor	a
	ld	(0x7000),a
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable
	ld	a,#0xaa
	ld	(0x7555),a		; write 0xaa to 0x1555
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
	ld	a,#1
	ld	(0x6000),a		; set winbond flash rom bank
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable
	ld	a,#0x55
	ld	(0x6aaa),a		; write 0x55 to 0x2aaa
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
	ld	a,#7
	ld	(0x6000),a		; set winbond flash rom bank
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable
   ;
	ld	a,#0xa0		; write flash command
	ld	(0x7555),a		; write command to winbond flash rom
	ld	a,#0x80		; set gameboy rom bank I/O write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
	ld	a,#2
	ld	(0x6000),a	; set current rom bank
	ld	a,#0xc0		; set flash rom write mode
	ld	(0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable

   ;;do the copy
.copyLoop:
	LD	A,(BC)		; point to Gameboy work Ram buffer (one page 256 bytes)
	ld	(HL),a		; point to Flash Rom address which page you want to write
   INC   BC
	inc	HL
	;
   dec   DE
   ld    A,D
   or    E
   jr    nz,.copyLoop ;DE!=0
   ;
        ld      a,#0xC4
        ld      (0xA000),a
        ; delay
        ld      de,#0x0A20
.wrLoop1:
		  dec     de
        ld      a,e
        cp      #0
        jr      nz,.wrLoop1
        ld      a,d
        cp      #0
        jr      nz,.wrLoop1
        ; end delay
        ld	    a,#0xC0		; set gameboy rom bank I/O write mode
        ld      (0xa000),a	; (W)d2=1 Eop, d1=1 record, d0=1 play, d6=1 Flash Rom write enable (0 for rom bank I/O)
        ld      a, #0x80
        ld      (0xA000),a

   ;
   POP   DE
	POP	BC
	RET
_end_PVoiceROMcpy::
